// -*- C++ -*-
/*!
 *
 * @file BufferBase.h
 * @brief Buffer abstract class
 * @date $Date: 2007-12-31 03:06:12 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: BufferBase.h 1704 2010-01-20 02:30:54Z fsi-katami $
 */

#ifndef RTC_BUFFERBASE_H
#define RTC_BUFFERBASE_H

#include <stddef.h>
#include <coil/Properties.h>
#include <rtm/BufferStatus.h>

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTR|[lg
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  /*!
   * @if jp
   * @class BufferBase
   * @brief BufferBase ۃNX
   * 
   * X̃obt@̂߂̒ۃC^[tF[XNXB
   * ۃobt@NX́Aȉ̏z֐̎񋟂Ȃ΂ȂȂB
   * \<DataType\>Ƃăobt@ŕێf[^^w肷B
   *
   * publicC^[tF[XƂĈȉ̂̂񋟂B
   * - length(): obt@̒Ԃ
   * - length(n): obt@nɃZbg
   * - reset(): obt@̃|C^Zbg
   *
   * ݊֘A
   * - wptr(n=0): ݂̏ݑΏۂ̗vfn̃|C^ԂB
   * - advanceWptr(n=1): ݃|C^ni߂B
   * - put(): ݂݈̏ʒuɏށA|C^͐i߂ȂB
   * - write(): obt@ɏށB|C^1ށB
   * - writable(): ݉\ȗvfԂB
   * - full(): obt@tԁB
   *
   * ǂݏo֘A
   * - rptr(n=0): ݂̓ǂݏoΏۂn̃|C^ԂB
   * - advanceRptr(n=1): ǂݏo|C^ni߂B
   * - get(): ݂̓ǂݏoʒuǂށB|C^͐i߂ȂB
   * - read(): obt@ǂݏoB|C^1ށB
   * - readable(): ǂݏo\vfԂB
   * - empty(): obt@ԁB
   *
   * @param DataType obt@Ɋi[f[^^
   *
   * @since 0.4.0
   *
   * @else
   * @class BufferBase
   * @brief BufferBase abstract class
   *
   * This is the abstract interface class for various Buffer.
   * Concrete buffer classes must implement the following pure virtual
   * functions.
   * The users specify data type to hold it in a buffer as \<DataType\>.
   *
   * This class provides public interface as follows.
   * - write(): Write data into the buffer.
   * - read(): Read data from the buffer.
   * - length(): Get the buffer length.
   * - isFull(): Check on whether the buffer is full.
   * - isEmpty(): Check on whether the buffer is empty.
   *
   * This class provides protected interface as follows.
   * - put(): Store data into the buffer.
   * - get(): Get data from the buffer.
   *
   * @param DataType Data type to be stored to the buffer.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  class BufferBase
    : public BufferStatus
  {
  public:
    BUFFERSTATUS_ENUM

    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * @else
     *
     * @brief Virtual destructor
     *
     * @endif
     */
    virtual ~BufferBase(void)
    {
    };
    
    /*!
     * @if jp
     *
     * @brief obt@̐ݒ
     *
     * @else
     *
     * @brief Set the buffer
     *
     * @endif
     */
    virtual void init(const coil::Properties& prop) = 0;

    /*!
     * @if jp
     *
     * @brief obt@̒擾
     * 
     * obt@擾邽߂̏z֐
     * 
     * @return obt@
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * Pure virtual function to get the buffer length.
     *
     * @return buffer length
     * 
     * @endif
     */
    virtual size_t length(void) const = 0;

    /*!
     * @if jp
     *
     * @brief obt@̒Zbg
     * 
     * obt@ݒ肷BݒsȏꍇNOT_SUPPORTEDԂB
     * 
     * @return BUFFER_OK: I
     *         NOT_SUPPORTED: obt@ύXs
     *         BUFFER_ERROR: ُI
     * 
     * @else
     *
     * @brief Set the buffer length
     *
     * Pure virtual function to set the buffer length.
     *
     * @return BUFFER_OK: Successful
     *         NOT_SUPPORTED: The buffer length cannot be set.
     *         BUFFER_ERROR: Failed
     *         
     * 
     * @endif
     */    
    virtual ReturnCode length(size_t n) = 0;

    /*!
     * @if jp
     *
     * @brief obt@̏ԂZbg
     * 
     * obt@̓ǂݏo|C^Ə݃|C^̈ʒuZbgB
     * 
     * @return BUFFER_OK: I
     *         NOT_SUPPORTED: Zbgs\
     *         BUFFER_ERROR: ُI
     * 
     * @else
     *
     * @brief Reset the buffer status
     *
     * Pure virtual function to reset the buffer status.
     *
     * @return BUFFER_OK: Successful
     *         NOT_SUPPORTED: The buffer status cannot be reset.
     *         BUFFER_ERROR: Failed
     * 
     * @endif
     */ 
    virtual ReturnCode reset() = 0;


    //----------------------------------------------------------------------
    /*!
     * @if jp
     *
     * @brief obt@݂̌̏ݗvf̃|C^
     * 
     * obt@݂̌̏ݗvf̃|C^܂́An̃|C^Ԃ
     * 
     * @param  n ݃|C^ + n ̈ʒũ|C^ 
     * @return ݈ʒũ|C^
     * 
     * @else
     *
     * @brief Get the writing pointer
     *
     * Pure virtual function to get the writing pointer.
     *
     * @param writeing pinter or n previous pointer
     * @return writing pointer
     * 
     * @endif
     */ 
    virtual DataType* wptr(long int n = 0) = 0;

    /*!
     * @if jp
     *
     * @brief ݃|C^i߂
     * 
     * ݂݈̏ʒũ|C^ n i߂B
     * 
     * @param  n ݃|C^ + n ̈ʒũ|C^ 
     * @return BUFFER_OK: I
     *         BUFFER_ERROR: ُI
     * 
     * @else
     *
     * @brief Forward n writing pointers.
     *
     * Pure virtual function to forward n writing pointers.
     *
     * @return BUFFER_OK: Successful
     *         BUFFER_ERROR: Failed
     * 
     * @endif
     */ 
    virtual ReturnCode advanceWptr(long int n = 1) = 0;

    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^
     * 
     * obt@Ƀf[^ށB݃|C^̈ʒu͕ύXȂB
     * 
     * @param value ݑΏۃf[^
     *
     * @return BUFFER_OK: I
     *         BUFFER_ERROR: ُI
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Pure virtual function to write data into the buffer.
     *
     * @param value Target data to write.
     *
     * @return BUFFER_OK: Successful
     *         BUFFER_ERROR: Failed
     *
     * @endif
     */
    virtual ReturnCode put(const DataType& value) = 0;

    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^
     * 
     * obt@Ƀf[^ށB݃|C^̈ʒu1ށB
     * 
     * @param value ݑΏۃf[^
     *
     * @return BUFFER_OK: I
     *         BUFFER_ERROR: ُI
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Pure virtual function to write data into the buffer.
     *
     * @param value Target data to write.
     *
     * @return BUFFER_OK: Successful
     *         BUFFER_ERROR: Failed
     *
     * @endif
     */
    virtual ReturnCode write(const DataType& value,
                             long int sec = -1, long int nsec = -1) = 0;

    /*!
     * @if jp
     *
     * @brief obt@ɏ݉\ȗvf
     * 
     * obt@ɏ݉\ȗvfԂB
     * 
     * @return ݉\ȗvf
     *
     * 
     * @else
     *
     * @brief Get a writable number. 
     *
     * Pure virtual function to get a writable number.
     *
     * @return value writable number
     *
     *
     * @endif
     */
    virtual size_t writable() const = 0;

    /*!
     * @if jp
     *
     * @brief obt@full`FbN
     * 
     * obt@full`FbNpz֐
     *
     * @return full`FbN(true:obt@fullCfalse:obt@󂫂)
     * 
     * @else
     *
     * @brief Check on whether the buffer is full.
     *
     * Pure virtual function to check on whether the buffer is full.
     *
     * @return True if the buffer is full, else false.
     *
     * @endif
     */
    virtual bool full(void) const = 0;

    //----------------------------------------------------------------------
    /*!
     * @if jp
     *
     * @brief obt@݂̌̓ǂݏovf̃|C^
     * 
     * obt@݂̌̓ǂݏovf̃|C^܂́An̃|C^Ԃ
     * 
     * @param  n ǂݏo|C^ + n ̈ʒũ|C^ 
     * @return ǂݏoʒũ|C^
     * 
     * @else
     *
     * @brief Get the reading pointer
     *
     * Pure virtual function to get the reading pointer.
     *
     * @return reading pointer
     * 
     * @endif
     */ 
    virtual DataType* rptr(long int n = 0) = 0;

    /*!
     * @if jp
     *
     * @brief ǂݏo|C^i߂
     * 
     * ݂̓ǂݏoʒũ|C^ n i߂B
     * 
     * @param  n ǂݏo|C^ + n ̈ʒũ|C^ 
     * @return BUFFER_OK: I
     *         BUFFER_ERROR: ُI
     * 
     * @else
     *
     * @brief Forward n reading pointers.
     *
     * Pure virtual function to forward n reading pointers.
     *
     * @return BUFFER_OK: Successful
     *         BUFFER_ERROR: Failed
     * 
     * @endif
     */ 
    virtual ReturnCode advanceRptr(long int n = 1) = 0;

    /*!
     * @if jp
     *
     * @brief obt@f[^ǂݏo
     * 
     * obt@f[^ǂ݂Bǂݏo|C^̈ʒu͕ύXȂB
     * 
     * @param value ǂݏof[^
     *
     * @return BUFFER_OK: I
     *         BUFFER_ERROR: ُI
     * 
     * @else
     *
     * @brief Read data from the buffer
     *
     * Pure virtual function to read data form the buffer.
     *
     * @param value Data to read.
     *
     * @return BUFFER_OK: Successful
     *         BUFFER_ERROR: Failed
     *
     * @endif
     */
    virtual ReturnCode get(DataType& value) = 0;

    /*!
     * @if jp
     *
     * @brief obt@f[^ǂݏo
     * 
     * obt@f[^ǂ݂Bǂݏo|C^̈ʒu͕ύXȂB
     * 
     * @return ǂݏof[^
     * 
     * @else
     *
     * @brief Read data from the buffer
     *
     * Pure virtual function to read data from the buffer.
     *
     * @return Data got from buffer.
     *
     * @endif
     */
    virtual DataType&  get() = 0;

    /*!
     * @if jp
     *
     * @brief obt@f[^ǂݏo
     * 
     * obt@f[^ǂݏo߂̏z֐
     * 
     * @param value ǂݏof[^
     *
     * @return f[^ǂݏo(true:ǂݏoCfalse:ǂݏos)
     * 
     * @else
     *
     * @brief Read data from the buffer
     *
     * Pure virtual function to read data from the buffer.
     *
     * @param value Read data.
     *
     * @return Result of having read (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual ReturnCode read(DataType& value,
                            long int sec = -1, long int nsec = -1) = 0;
    
    /*!
     * @if jp
     *
     * @brief obt@ǂݏo\ȗvf
     * 
     * obt@ǂݏo\ȗvfԂB
     * 
     * @return ǂݏo\ȗvf
     *
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Pure virtual function to get a reading number.
     *
     *
     * @return readable number
     *
     * @endif
     */
    virtual size_t readable() const = 0;

    /*!
     * @if jp
     *
     * @brief obt@empty`FbN
     * 
     * obt@empty`FbNpz֐
     *
     * @return empty`FbN(true:obt@emptyCfalse:obt@f[^)
     * 
     * @else
     *
     * @brief Check on whether the buffer is empty.
     *
     * Pure virtual function to check on whether the buffer is empty.
     *
     * @return True if the buffer is empty, else false.
     *
     * @endif
     */
    virtual bool empty(void) const = 0;

  };
  
  /*!
   * @if jp
   * @class NullBuffer
   * @brief _~[obt@NX
   * 
   * obt@PŒ̃_~[obt@NXB
   * \<DataType\>Ƃăobt@ŕێf[^^w肷B
   *
   * @param DataType obt@Ɋi[f[^^
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class NullBuffer
   * @brief Concrete buffer class for dummy
   * 
   * Concrete buffer class for dummy. Buffer length is fixed to 1.
   * The users specify data type to hold it in a buffer as \<DataType\>.
   *
   * @param DataType Data type to hold in a buffer
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  class NullBuffer
    : public BufferBase<DataType>
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     * 
     * RXgN^
     * obt@1(Œ)ŏB
     *
     * @param size obt@()
     * 
     * @else
     *
     * @brief Constructer
     * 
     * Constructer.
     * Initialize buffer length to always 1.
     *
     * @param size Buffer length(Not use)
     * 
     * @endif
     */
    NullBuffer(long int size = 1)
      : m_length(1)
    {
    }
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     * 
     * fXgN^B
     *
     * @else
     *
     * @brief Destructor
     *
     * Destractor
     *
     * @endif
     */
    virtual ~NullBuffer(void)
    {
    }
    
    /*!
     * @if jp
     *
     * @brief obt@(1Œ)擾
     * 
     * obt@擾B(1ԂB)
     * 
     * @return obt@(1Œ)
     * 
     * @else
     *
     * @brief Get the buffer length (always 1)
     *
     * Get the buffer length. (Return always 1.)
     *
     * @return buffer length(always 1)
     * 
     * @endif
     */
    virtual long int length(void) const
    {
      return 1;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^
     * 
     * ŗ^ꂽf[^obt@ɏށB
     * 
     * @param value ݑΏۃf[^
     *
     * @return f[^݌(true:ݐCfalse:ݎs)
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Write data which were given with an argument into the buffer.
     *
     * @param value Target data to write.
     *
     * @return Result of having written in data (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual bool write(const DataType& value)
    {
      m_data = value;
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@f[^ǂݏo
     * 
     * obt@Ɋi[ꂽf[^ǂݏoB
     * 
     * @param value ǂݏof[^
     *
     * @return f[^ǂݏo(true:ǂݏoCfalse:ǂݏos)
     * 
     * @else
     *
     * @brief Read data from the buffer
     *
     * Read data stored in the buffer.
     *
     * @param value Read data.
     *
     * @return Result of having read (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual bool read(DataType& value)
    {
      value = m_data;
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@full`FbN
     * 
     * obt@full`FbNB(falseԂB)
     *
     * @return full`FbN(false)
     * 
     * @else
     *
     * @brief Check on whether the buffer is full.
     *
     * Check on whether the buffer is full. (Always false.)
     *
     * @return Always false.
     *
     * @endif
     */
    virtual bool isFull(void) const
    {
      return false;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@empty`FbN
     * 
     * obt@empty`FbNB(falseԂB)
     *
     * @return empty`FbN(false)
     * 
     * @else
     *
     * @brief Check on whether the buffer is empty.
     *
     * Check on whether the buffer is empty. (Always false.)
     *
     * @return Always false.
     *
     * @endif
     */
    virtual bool isEmpty(void) const
    {
      return false;
    }
    
  protected:
    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^i[
     * 
     * ŗ^ꂽf[^obt@Ɋi[B
     * 
     * @param data Ώۃf[^
     * 
     * @else
     *
     * @brief Store data into the buffer
     *
     * Store data which were given with an argument into the buffer.
     *
     * @param data Target data to store.
     *
     * @endif
     */
    virtual void put(const DataType& data)
    {
      m_data = data;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@f[^擾
     * 
     * obt@Ɋi[ꂽf[^擾B
     *
     * @return 擾f[^
     * 
     * @else
     *
     * @brief Get data from the buffer
     *
     * Get data from the buffer.
     *
     * @return Data got from buffer.
     *
     * @endif
     */
    virtual const DataType& get(void)
    {
      return m_data;
    }
    
    /*!
     * @if jp
     *
     * @brief ɏރobt@ւ̎QƂ擾
     * 
     * ݃obt@ւ̎QƂ擾B
     * {obt@ł̓obt@͌ŒłPł邽߁C
     * ɓʒuւ̎QƂԂB
     *
     * @return ̏ݑΏۃobt@ւ̎Q(Œ)
     * 
     * @else
     *
     * @brief Get the buffer's reference to be written the next
     *
     * Get the reference to be written buffer.
     * Return always same position because this buffer's length is always 1.
     *
     * @return Reference to be written the next(always same)
     *
     * @endif
     */
    virtual DataType& getRef(void)
    {
      return m_data;
    }
    
  private:
    DataType m_data;
    long int m_length;
  };
}; // namespace RTC
#endif // BufferBase_h
